@@ -32,6 +32,7 @@
 #include <limits.h>
 #include <stdlib.h>
 #include <errno.h>
+#include <inttypes.h>
 #include "gettext.h"
 #include <linux/fs.h>
 #define _(STRING) gettext(STRING)
@@ -70,7 +71,7 @@ extern void print_readable_size_str(unsigned long long size_byte, char *new_size
     uint64_t kbyte=1000;
     
     if (size_byte == 0)
-	snprintf(new_size_str, 11, "%lli", size_byte);
+	snprintf(new_size_str, 11, "%llu", size_byte);
 
     if (size_byte >= tbyte){
 	new_size = (float)size_byte / (float)tbyte;
@@ -126,6 +127,7 @@ extern void usage(void)
             "    -m,  --max_block_cache  The used block will be cache until max number\n"
             "    -q,  --quiet		 Disable progress message\n"
             "    -B,  --no_block_detail  Show progress message without block detail\n"
+            "    -E,  --offset=X	 Add offset X (bytes) to OUTPUT\n"
             "    -v,  --version          Display partclone version\n"
             "    -h,  --help             Display this help\n"
             , EXECNAME, VERSION, EXECNAME);
@@ -143,7 +145,7 @@ enum {
 
 extern void parse_options(int argc, char **argv, cmd_opt* opt)
 {
-    static const char *sopt = "-hvd::L:cbrDo:O:s:f:m:RCXFINiqWB";
+    static const char *sopt = "-hvd::L:cbrDo:O:s:f:m:RCXFINiqWBE:";
     static const struct option lopt[] = {
         { "help",		no_argument,	    NULL,   'h' },
         { "print_version",	no_argument,	    NULL,   'v' },
@@ -166,6 +168,7 @@ extern void parse_options(int argc, char **argv, cmd_opt* opt)
 	{ "ignore_crc",		no_argument,	    NULL,   'i' },
         { "force",		no_argument,	    NULL,   'F' },
         { "quiet",		no_argument,	    NULL,   'q' },
+        { "offset",		required_argument,  NULL,   'E' },
         { "no_block_detail",	no_argument,	    NULL,   'B' },
 #ifdef HAVE_LIBNCURSESW
         { "ncurses",		no_argument,	    NULL,   'N' },
@@ -177,6 +180,7 @@ extern void parse_options(int argc, char **argv, cmd_opt* opt)
     int mode = 0;
     memset(opt, 0, sizeof(cmd_opt));
     opt->debug = 0;
+    opt->offset = 0;
     opt->rescue = 0;
     opt->max_block_cache = 8;
     opt->check = 1;
@@ -258,7 +262,10 @@ extern void parse_options(int argc, char **argv, cmd_opt* opt)
             case 'B':
                 opt->no_block_detail = 1;
                 break;
-            case 'R':
+            case 'E':
+                opt->offset = atol(optarg);
+                break;
+	    case 'R':
                 opt->rescue++;
                 break;
 #ifdef HAVE_LIBNCURSESW
@@ -333,12 +340,11 @@ extern void parse_options(int argc, char **argv, cmd_opt* opt)
  * close_ncurses    - close text window
  */
 extern int open_ncurses(){
+#ifdef HAVE_LIBNCURSESW
     int debug = 1;
 
-#ifdef HAVE_LIBNCURSESW
     FILE *in = fopen( "/dev/stderr", "r" );
     FILE *out = fopen( "/dev/stderr", "w" );
-    extern cmd_opt opt;
     int terminal_x = 0;
     int terminal_y = 0;
 
@@ -483,7 +489,7 @@ extern void log_mesg(int log_level, int log_exit, int log_stderr, int debug, con
     extern cmd_opt opt;
     char tmp_str[512];
 
-    vsprintf(tmp_str, fmt, args);
+    vsnprintf(tmp_str, sizeof(tmp_str), fmt, args);
     if (opt.ncurses) {
 #ifdef HAVE_LIBNCURSESW
         setlocale(LC_ALL, "");
@@ -550,6 +556,8 @@ extern void restore_image_hdr(int* ret, cmd_opt* opt, image_head* image_hdr){
     int debug = opt->debug;
 
     buffer = (char*)malloc(sizeof(image_head));
+    if (buffer == NULL)
+        log_mesg(0, 1, 1, debug, "%s, %i, ERROR:%s", __func__, __LINE__, strerror(errno));
     memset(buffer, 0, sizeof(image_head));
     r_size = read_all(ret, buffer, sizeof(image_head), opt);
     if (r_size == -1)
@@ -567,6 +575,8 @@ extern void restore_image_hdr_sp(int* ret, cmd_opt* opt, image_head* image_hdr,
     int debug = opt->debug;
 
     buffer = (char*)malloc(sizeof(image_head));
+    if (buffer == NULL)
+        log_mesg(0, 1, 1, debug, "%s, %i, ERROR:%s", __func__, __LINE__, strerror(errno));
     memset(buffer, 0, sizeof(image_head));
     memcpy(buffer, first_sec, 512);
     r_size = read_all(ret, (buffer+512), (sizeof(image_head)-512), opt);
@@ -597,7 +607,7 @@ extern unsigned long long get_partition_size(int* ret){
             if (ioctl(*ret, BLKGETSIZE64, &dest_size) < 0) {
                 log_mesg(0, 0, 0, debug, "get device size error, Use option -C to disable size checking(Dangerous).\n");
             }
-            log_mesg(1, 0, 0, debug, "get device size %lli by ioctl BLKGETSIZE64,\n", dest_size);
+            log_mesg(1, 0, 0, debug, "get device size %llu by ioctl BLKGETSIZE64,\n", dest_size);
             return dest_size;
 #endif
 
@@ -605,7 +615,7 @@ extern unsigned long long get_partition_size(int* ret){
             if (ioctl(*ret, BLKGETSIZE, &dest_block) >= 0) {
                 dest_size = (unsigned long long)(dest_block * 512);
             }
-            log_mesg(1, 0, 0, debug, "get block %li and device size %lli by ioctl BLKGETSIZE,\n", dest_block, dest_size);
+            log_mesg(1, 0, 0, debug, "get block %lu and device size %llu by ioctl BLKGETSIZE,\n", dest_block, dest_size);
             return dest_size;
 #endif
         }
@@ -625,7 +635,7 @@ extern int check_size(int* ret, unsigned long long size){
 
     dest_size = get_partition_size(ret);
     if (dest_size < size){
-        log_mesg(0, 1, 1, debug, "Target partition size(%lli MB) is smaller than source(%lli MB). Use option -C to disable size checking(Dangerous).\n", print_size(dest_size, MBYTE), print_size(size, MBYTE));
+        log_mesg(0, 1, 1, debug, "Target partition size(%llu MB) is smaller than source(%llu MB). Use option -C to disable size checking(Dangerous).\n", print_size(dest_size, MBYTE), print_size(size, MBYTE));
         return 1;
     }
 
@@ -674,7 +684,7 @@ extern int check_mem_size(image_head image_hdr, cmd_opt opt, unsigned long long
     bitmap_size = sizeof(unsigned long)*LONGS(image_hdr.totalblock);
     crc_io_size = CRC_SIZE+image_hdr.block_size;
     *mem_size = image_head_size + bitmap_size + crc_io_size;
-    log_mesg(0, 0, 0, 1, "we need memory: %lld bytes\nimage head %lld, bitmap %lld, crc %i bytes\n", *mem_size, image_head_size, bitmap_size, crc_io_size);
+    log_mesg(0, 0, 0, 1, "we need memory: %llu bytes\nimage head %llu, bitmap %llu, crc %i bytes\n", *mem_size, image_head_size, bitmap_size, crc_io_size);
 
     test_mem = malloc(*mem_size);
     if (test_mem == NULL){
@@ -722,7 +732,7 @@ extern void get_image_bitmap(int* ret, cmd_opt opt, image_head image_hdr, unsign
                 err_exit = 0;
             else
                 err_exit = 1;
-            log_mesg(0, err_exit, 1, debug, "The Used Block count is different.(bitmap %lli != image_head %lli)\nTry to use --force to skip the metadata error.\n", bused, image_hdr.usedblocks);
+            log_mesg(0, err_exit, 1, debug, "The Used Block count is different.(bitmap %llu != image_head %llu)\nTry to use --force to skip the metadata error.\n", bused, image_hdr.usedblocks);
         }	
     } 
 }
@@ -750,13 +760,12 @@ extern int check_mount(const char* device, char* mount_p){
 
     real_file = malloc(PATH_MAX + 1);
     if (!real_file){
-	free(real_file);
         return -1;
     }
 
     real_fsname = malloc(PATH_MAX + 1);
     if (!real_fsname){
-	free(real_fsname);
+	free(real_file);
         return -1;
         //err = errno;
     }
@@ -797,6 +806,8 @@ extern int open_source(char* source, cmd_opt* opt){
     if((opt->clone) || (opt->dd) || (opt->domain)){ /// always is device, clone from device=source
 
         mp = malloc(PATH_MAX + 1);
+        if (mp == NULL)
+            log_mesg(0, 1, 1, debug, "%s, %i, ERROR:%s", __func__, __LINE__, strerror(errno));
         if (check_mount(source, mp) == 1){
             log_mesg(0, 0, 1, debug, "device (%s) is mounted at %s\n", source, mp);
 	    free(mp);
@@ -855,6 +866,8 @@ extern int open_target(char* target, cmd_opt* opt){
 
         /// check mounted
         mp = malloc(PATH_MAX + 1);
+        if (mp == NULL)
+            log_mesg(0, 1, 1, debug, "%s, %i, ERROR:%s", __func__, __LINE__, strerror(errno));
         if (check_mount(target, mp) == 1){
             log_mesg(0, 0, 1, debug, "device (%s) is mounted at %s\n", target, mp);
 	    free(mp);
@@ -907,7 +920,7 @@ extern int io_all(int *fd, char *buf, unsigned long long count, int do_write, cm
         } else {
             count -= i;
             buf = i + (char *) buf;
-            log_mesg(2, 0, 0, debug, "%s: read %lli, %lli left.\n",__func__, i, count);
+            log_mesg(2, 0, 0, debug, "%s: read %lli, %llu left.\n",__func__, i, count);
         }
     }
     return size;
@@ -926,14 +939,14 @@ extern void rescue_sector(int *fd, unsigned long long pos, char *buff, cmd_opt *
     const char *badsector_magic = "BADSECTOR\0";
 
     if (lseek(*fd, pos, SEEK_SET) == (off_t)-1){
-        log_mesg(0, 0, 1, opt->debug, "WARNING: lseek error at %lli\n", pos);
+        log_mesg(0, 0, 1, opt->debug, "WARNING: lseek error at %llu\n", pos);
         memset(buff, '?', SECTOR_SIZE);
         memmove(buff, badsector_magic, sizeof(badsector_magic));
 	return;
     }
 
     if (io_all(fd, buff, SECTOR_SIZE, 0, opt) == -1) { /// read_all
-        log_mesg(0, 0, 1, opt->debug, "WARNING: Can't read sector at %llu, lost data.\n", (unsigned long long)pos);
+        log_mesg(0, 0, 1, opt->debug, "WARNING: Can't read sector at %llu, lost data.\n", pos);
         memset(buff, '?', SECTOR_SIZE);
         memmove(buff, badsector_magic, sizeof(badsector_magic));
     }
@@ -1039,7 +1052,6 @@ extern void print_image_hdr_info(image_head image_hdr, cmd_opt opt){
     int block_s  = image_hdr.block_size;
     unsigned long long total    = image_hdr.totalblock;
     unsigned long long used     = image_hdr.usedblocks;
-    unsigned long long dev_size = image_hdr.device_size;
     int debug = opt.debug;
     char size_str[11];
 
@@ -1049,16 +1061,16 @@ extern void print_image_hdr_info(image_head image_hdr, cmd_opt opt){
     log_mesg(0, 0, 1, debug, _("File system:  %s\n"), image_hdr.fs);
 
     print_readable_size_str(total*block_s, size_str);
-    log_mesg(0, 0, 1, debug, _("Device size:  %s = %lld Blocks\n"), size_str, total, (unsigned long long)(total*block_s));
+    log_mesg(0, 0, 1, debug, _("Device size:  %s = %llu Blocks\n"), size_str, total);
     
     print_readable_size_str(used*block_s, size_str);
-    log_mesg(0, 0, 1, debug, _("Space in use: %s = %lld Blocks\n"), size_str, used, (unsigned long long)(used*block_s));
+    log_mesg(0, 0, 1, debug, _("Space in use: %s = %llu Blocks\n"), size_str, used);
     
     print_readable_size_str((total-used)*block_s, size_str);
-    log_mesg(0, 0, 1, debug, _("Free Space:   %s = %lld Blocks\n"), size_str, (total-used), (unsigned long long)((total-used)*block_s));
+    log_mesg(0, 0, 1, debug, _("Free Space:   %s = %llu Blocks\n"), size_str, (total-used));
     
     log_mesg(0, 0, 1, debug, _("Block size:   %i Byte\n"), block_s);
-    //log_mesg(0, 0, 1, debug, _("Used block :  %lli\n"), used);
+    //log_mesg(0, 0, 1, debug, _("Used block :  %llu\n"), used);
 }
 
 /// print finish message
@@ -1108,12 +1120,8 @@ extern void initial_dd_hdr(int ret, image_head* image_hdr){
 
 /// initial bitmap
 extern void dd_bitmap(image_head image_hdr, unsigned long* bitmap){
-
-    int block;
-
     /// initial image bitmap as 1 (all block are used)
     memset(bitmap, 0xFF, sizeof(unsigned long)*LONGS(image_hdr.totalblock));
-
 }
 
 /// write last block
@@ -1124,7 +1132,10 @@ void write_last_block(int* dfw, int size, unsigned long long id, cmd_opt* opt){
 
     sf = lseek(*dfw, (size*id), SEEK_SET);
     buffer = (char*)malloc(size);
+    if (buffer == NULL)
+        log_mesg(0, 1, 1, opt->debug, "%s, %i, ERROR:%s", __func__, __LINE__, strerror(errno));
     memset(buffer, 0, size);
     st = io_all(dfw, buffer, size, 1, opt);
-    log_mesg(1, 0, 0, opt->debug, "write last block%lli, size %i ,status %i\n", id, size, st);
+    free(buffer);
+    log_mesg(1, 0, 0, opt->debug, "write last block%llu, size %i ,status %ji\n", id, size, (intmax_t)st);
 }
